@@ -5,9 +5,9 @@ from statham.dsl.elements.composition import (
     AllOf,
     AnyOf,
     CompositionElement,
+    Not,
     OneOf,
 )
-from statham.dsl.elements.meta import ObjectOptions
 from statham.dsl.elements.null import Null
 from statham.dsl.elements.numeric import Integer, Number
 from statham.dsl.elements.object import Object
