@@ -1,17 +1,7 @@
-from typing import List, Union
+from typing import List
 
 from statham.dsl.constants import Maybe
-from statham.dsl.elements import (
-    AnyOf,
-    Array,
-    Boolean,
-    Integer,
-    Null,
-    Number,
-    OneOf,
-    Object,
-    String,
-)
+from statham.dsl.elements import Array, Object, String
 from statham.dsl.property import Property
 
 
